# -*- coding: utf-8 -*-
#    Script to calculate linear trends from Jones et al., 1991 dataset
#    Copyright (C) 2013 Dr. Ronan Connolly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#
#    ********************************************************************
#

#    This script uses the Jones et al., 1991 gridded temperature
#    dataset, archived as NDP020 by CDIAC at
#    http://cdiac.ornl.gov/ftp/ndp020/
#    see http://dx.doi.org/10.3334/CDIAC/cli.ndp020

import numpy as np
import matplotlib.pyplot as plt
import os
from scipy import stats
from mpl_toolkits.basemap import Basemap 

start_years=[1901,1930,1954,1901]
end_years=[1987,1987,1983,1984]
Jones_study=["Western USSR 1","Western USSR 2", "Eastern China", "Contiguous US"]

western_USSR=[(70,40),(70,50),
              (65,30),(65,40),(65,50),(65,60),(65,70),
              (60,30),(60,40),(60,50),(60,60),(60,70),
              (55,30),(55,40),(55,50),(55,60),(55,70),
              (50,30),(50,40),(50,50),(50,60),(50,70)]
eastern_Australia=[(-15,140),(-15,150),(-20,140),(-20,150),
                   (-25,140),(-25,150),(-30,140),(-30,150),
                   (-35,140),(-35,150),(-40,140),(-40,150),
                   (-45,140),(-45,150)]
eastern_China=[(45,100),(45,110),(45,120),
               (40,100),(40,110),(40,120),
               (35,100),(35,110),(35,120),
               (30,100),(30,110),(30,120),
               (25,100),(25,110),(25,120)]
contiguous_US=[(50,-120),(50,-110),(50,-100),(50,-90),(50,-80),(50,-70),
               (45,-120),(45,-110),(45,-100),(45,-90),(45,-80),(45,-70),
               (40,-120),(40,-110),(40,-100),(40,-90),(40,-80),(40,-70),
               (35,-120),(35,-110),(35,-100),(35,-90),(35,-80),(35,-70),
               (30,-120),(30,-110),(30,-100),(30,-90),(30,-80),(30,-70)]

grid_anomalies={}
num_stations={} # number of stations in grid
mean_distance={} # average distance from center of gridbox
lat=-85
while lat<=85:
    grid_anomalies[lat]={}
    num_stations[lat]={}
    mean_distance[lat]={}
    lon=-170
    while lon<=180:
        grid_anomalies[lat][lon]={}
        num_stations[lat][lon]={}
        mean_distance[lat][lon]={}
        lon=lon+10
    lat=lat+5

input_folder="input_Jones90"
if not os.path.isdir(input_folder):
    os.mkdir(input_folder)

output_folder="output_Jones90"
if not os.path.isdir(output_folder):
    os.mkdir(output_folder)

if os.path.isfile(input_folder+"/nhem1990.ano"):
    nhem1990=open(input_folder+"/nhem1990.ano",'r')
    print "Loading Northern Hemisphere data..."
else:
    print "Cannot find nhem1990.ano"
    print "Download from http://dx.doi.org/10.3334/CDIAC/cli.ndp020"
    print "Then try again..."
    raise SystemExit

datatype="idist"
i=0
for k in nhem1990:
    if i==0:
        lon=0 # first line will start at 0E, heading east by 10 deg
        if datatype=="idist":
            datatype="anom"
            year=int(k[18:24])
            month=int(k[24:30])
        elif datatype=="anom":
            datatype="nsta"
        elif datatype=="nsta":
            datatype="idist"
    else:
        j=0
        while j<18:
            lat=(17-j)*5
            value=k[(j*6):(j*6)+6]
            if datatype=="anom":
                cur_anom=float(value)/100 # anom in deg C
                if cur_anom>-99:
                    if not(grid_anomalies[lat][lon].has_key(year)):
                        grid_anomalies[lat][lon][year]={}
                    grid_anomalies[lat][lon][year][month]=cur_anom
            elif datatype=="nsta":
                if not(num_stations[lat][lon].has_key(year)):
                    num_stations[lat][lon][year]={}
                num_stations[lat][lon][year][month]=int(value) # number of stations in grid
            elif datatype=="idist":
                if not(mean_distance[lat][lon].has_key(year)):
                    mean_distance[lat][lon][year]={}
                mean_distance[lat][lon][year][month]=float(value)/10000 # mean distance from centre
            j=j+1
        if lon==180:
            lon=-170
        else:
            lon=lon+10
    i=i+1
    if i>36:
        i=0
nhem1990.close()

if os.path.isfile(input_folder+"/shem1990.ano"):
    shem1990=open(input_folder+"/shem1990.ano",'r')
    print "Loading Southern Hemisphere data..."
else:
    print "Cannot find shem1990.ano"
    print "Download from http://dx.doi.org/10.3334/CDIAC/cli.ndp020"
    print "Then try again..."
    raise SystemExit

datatype="idist"
i=0
for k in shem1990:
    if i==0:
        lon=0 # first line will start at 0E, heading east by 10 deg
        if datatype=="idist":
            datatype="anom"
            year=int(k[18:24])
            month=int(k[24:30])
        elif datatype=="anom":
            datatype="nsta"
        elif datatype=="nsta":
            datatype="idist"
    else:
        j=0
        while j<12:
            lat=-5-(j*5)
            value=k[(j*6):(j*6)+6]
            if datatype=="anom":
                cur_anom=float(value)/100 # anom in deg C
                if cur_anom>-99:
                    if not(grid_anomalies[lat][lon].has_key(year)):
                        grid_anomalies[lat][lon][year]={}
                    grid_anomalies[lat][lon][year][month]=cur_anom
            elif datatype=="nsta":
                if not(num_stations[lat][lon].has_key(year)):
                    num_stations[lat][lon][year]={}
                num_stations[lat][lon][year][month]=int(value) # number of stations in grid
            elif datatype=="idist":
                if not(mean_distance[lat][lon].has_key(year)):
                    mean_distance[lat][lon][year]={}
                mean_distance[lat][lon][year][month]=float(value)/10000 # mean distance from centre
            j=j+1
        if lon==180:
            lon=-170
        else:
            lon=lon+10
    i=i+1
    if i>36:
        i=0
shem1990.close()

if os.path.isfile(input_folder+"/ant5790.ano"):
    antar1990=open(input_folder+"/ant5790.ano",'r')
    print "Loading Antarctica data..."
else:
    print "Cannot find ant5790.ano"
    print "Download from http://dx.doi.org/10.3334/CDIAC/cli.ndp020"
    print "Then try again..."
    raise SystemExit

datatype="idist"
i=0
for k in antar1990:
    if i==0:
        lon=0 # first line will start at 0E, heading east by 10 deg
        if datatype=="idist":
            datatype="anom"
            year=int(k[18:24])
            month=int(k[24:30])
        elif datatype=="anom":
            datatype="nsta"
        elif datatype=="nsta":
            datatype="idist"
    else:
        j=0
        while j<5:
            lat=-65-(j*5)
            value=k[(j*6):(j*6)+6]
            if datatype=="anom":
                cur_anom=float(value)/100 # anom in deg C
                if cur_anom>-99:
                    if not(grid_anomalies[lat][lon].has_key(year)):
                        grid_anomalies[lat][lon][year]={}
                    grid_anomalies[lat][lon][year][month]=cur_anom
            elif datatype=="nsta":
                if not(num_stations[lat][lon].has_key(year)):
                    num_stations[lat][lon][year]={}
                num_stations[lat][lon][year][month]=int(value) # number of stations in grid
            elif datatype=="idist":
                if not(mean_distance[lat][lon].has_key(year)):
                    mean_distance[lat][lon][year]={}
                mean_distance[lat][lon][year][month]=float(value)/10000 # mean distance from centre
            j=j+1
        if lon==180:
            lon=-170
        else:
            lon=lon+10
    i=i+1
    if i>36:
        i=0
antar1990.close()

print "Calculating regional trends"
regions=open(output_folder+"/regional_trends.txt",'w')
regions.write("Year\tWestern USSR\tEastern Australia\tEastern China\tContiguous U.S.\n")

year=1851
while year<=1990:
    regions.write(str(year)+"\t")
    w_USSR=[]
    weight_w_USSR=[]
    for coord in western_USSR:
        lat=coord[0]
        lon=coord[1]
        lat_weight=np.cos(np.radians(abs(lat) + 2.5))

        if grid_anomalies[lat][lon].has_key(year):
            monthly_anomalies=[]
            for month in grid_anomalies[lat][lon][year]:
                if grid_anomalies[lat][lon][year].has_key(month):
                    monthly_anomalies.append(grid_anomalies[lat][lon][year][month])
            if len(monthly_anomalies)>8:
                annual_mean=round(np.mean(monthly_anomalies),2)
                w_USSR.append(annual_mean*lat_weight)
                weight_w_USSR.append(lat_weight)
    if len(w_USSR)>0:
        gridded_mean=round(np.sum(w_USSR)/np.sum(weight_w_USSR),2)
        regions.write(str(gridded_mean)+"\t")
    else:
        regions.write("\t")
    e_Australia=[]
    weight_e_Australia=[]
    for coord in eastern_Australia:
        lat=coord[0]
        lon=coord[1]
        lat_weight=np.cos(np.radians(abs(lat) + 2.5))

        if grid_anomalies[lat][lon].has_key(year):
            monthly_anomalies=[]
            for month in grid_anomalies[lat][lon][year]:
                if grid_anomalies[lat][lon][year].has_key(month):
                    monthly_anomalies.append(grid_anomalies[lat][lon][year][month])
            if len(monthly_anomalies)>8:
                annual_mean=round(np.mean(monthly_anomalies),2)
                e_Australia.append(annual_mean*lat_weight)
                weight_e_Australia.append(lat_weight)
    if len(e_Australia)>0:
        gridded_mean=round(np.sum(e_Australia)/np.sum(weight_e_Australia),2)
        regions.write(str(gridded_mean)+"\t")
    else:
        regions.write("\t")
    e_China=[]
    weight_e_China=[]
    for coord in eastern_China:
        lat=coord[0]
        lon=coord[1]
        lat_weight=np.cos(np.radians(abs(lat) + 2.5))

        if grid_anomalies[lat][lon].has_key(year):
            monthly_anomalies=[]
            for month in grid_anomalies[lat][lon][year]:
                if grid_anomalies[lat][lon][year].has_key(month):
                    monthly_anomalies.append(grid_anomalies[lat][lon][year][month])
            if len(monthly_anomalies)>8:
                annual_mean=round(np.mean(monthly_anomalies),2)
                e_China.append(annual_mean*lat_weight)
                weight_e_China.append(lat_weight)
    if len(e_China)>0:
        gridded_mean=round(np.sum(e_China)/np.sum(weight_e_China),2)
        regions.write(str(gridded_mean)+"\t")
    else:
        regions.write("\t")
    USA=[]
    weight_USA=[]
    for coord in contiguous_US:
        lat=coord[0]
        lon=coord[1]
        lat_weight=np.cos(np.radians(abs(lat) + 2.5))

        if grid_anomalies[lat][lon].has_key(year):
            monthly_anomalies=[]
            for month in grid_anomalies[lat][lon][year]:
                if grid_anomalies[lat][lon][year].has_key(month):
                    monthly_anomalies.append(grid_anomalies[lat][lon][year][month])
            if len(monthly_anomalies)>8:
                annual_mean=round(np.mean(monthly_anomalies),2)
                USA.append(annual_mean*lat_weight)
                weight_USA.append(lat_weight)
    if len(USA)>0:
        gridded_mean=round(np.sum(USA)/np.sum(weight_USA),2)
        regions.write(str(gridded_mean)+"\n")
    else:
        regions.write("\n")
    year=year+1
    
regions.close()

# Generate maps & trends for different periods
loop_counter=0
while loop_counter<len(start_years):
    lats_cool=[]
    lons_cool=[]
    lats_cold=[]
    lons_cold=[]
    lats_warm=[]
    lons_warm=[]
    lats_hot=[]
    lons_hot=[]
    start_trend=start_years[loop_counter]
    end_trend=end_years[loop_counter]
    print "Calculating linear trends from "+str(start_trend)+" to "+str(end_trend)+". "+Jones_study[loop_counter]
    
    gridded_trends=open(output_folder+"/gridded_trends_"+str(start_trend)+"-"+str(end_trend)+".txt",'w')
    gridded_trends.write("Lat v, Lon->\t")
    lon=-170
    while lon<=180:
        gridded_trends.write(str(lon)+"\t")
        lon=lon+10
    gridded_trends.write("\n")
    
    for lat in sorted(grid_anomalies):
        gridded_trends.write(str(lat)+"\t")
        for lon in sorted(grid_anomalies[lat]):
            grid_values=[]
            grid_years=[]
            year=start_trend
            while year<=end_trend:
                if grid_anomalies[lat][lon].has_key(year):
                    monthly_anomalies=[]
                    for month in grid_anomalies[lat][lon][year]:
                        if grid_anomalies[lat][lon][year][month]>-90:
                            monthly_anomalies.append(grid_anomalies[lat][lon][year][month])
                    if len(monthly_anomalies)>2:
                        annual_mean=np.mean(monthly_anomalies)
                        grid_years.append(year)
                        grid_values.append(annual_mean)
                year=year+1
            
            if len(grid_years)>3:
                slope, intercept, r_value, p_value, std_err = stats.linregress(grid_years,grid_values)
                gridded_trends.write(str(round(slope,3))+"\t")
                if slope>0:
                    if slope>=0.02:
                        lats_hot.append(lat)
                        lons_hot.append(lon)
                    else:
                        lats_warm.append(lat)
                        lons_warm.append(lon)
                else:
                    if slope<=-0.02:
                        lats_cold.append(lat)
                        lons_cold.append(lon)
                    else:
                        lats_cool.append(lat)
                        lons_cool.append(lon)
            else:
                gridded_trends.write("\t")
        gridded_trends.write("\n")
    gridded_trends.close()
    
    plt.clf()
    m = Basemap(projection='robin',lon_0=0,resolution='c')
    m.drawcoastlines()
    m.fillcontinents(color='white',lake_color='dodgerblue')
    # draw parallels and meridians.
    m.drawparallels(np.arange(-90.,120.,30.))
    m.drawmeridians(np.arange(0.,360.,60.))
    m.drawmapboundary(fill_color='dodgerblue')
    plt.title(str(start_trend)+"-"+str(end_trend), fontsize=24)
    
    x,y = m(lons_cold,lats_cold)
    m.plot(x,y, linestyle='none',color='cyan', marker='D',markersize=5)
    x,y = m(lons_cool,lats_cool)
    m.plot(x,y, linestyle='none',color='cyan', marker='D',markersize=3, label="Cooling")
    x,y = m(lons_hot,lats_hot)
    m.plot(x,y, linestyle='none',color='red', marker='o',markersize=5)
    x,y = m(lons_warm,lats_warm)
    m.plot(x,y, linestyle='none',color='red', marker='o',markersize=3, label="Warming")

    plt.legend(bbox_to_anchor=(0.25,0.25),numpoints=1)
    plt.savefig(output_folder+'/linear_trends_map_'+str(start_trend)+"-"+str(end_trend)+'.png')

    loop_counter=loop_counter+1

print "Done."
